@@ -18,9 +18,6 @@ use strict;
 
 package DBI::Format::SQLMinus;
 
-our $VERSION = '11.96_03'; # TRIAL VERSION
-$VERSION = eval $VERSION;
-
 @DBI::Format::SQLMinus::ISA = qw(DBI::Format::Base);
 
 use Text::Abbrev;
@@ -28,11 +25,17 @@ use Text::Reform qw(form break_with);
 
 use Data::Dumper;
 
+$DBI::Format::SQLMinus::VERSION =
+	$DBI::Format::SQLMinus::VERSION =
+		(qw$Revision: 11.8 $)[1];
+
 sub header {
-    my ($self, $sth, $fh, $sep) = @_;
-	$self->SUPER::header($sth, $fh, $sep);
+    my($self, $sth, $fh, $sep) = @_;
+    $self->{'fh'} = $self->setup_fh($fh);
+    $self->{'sth'} = $sth;
     $self->{'data'} = [];
     $self->{'formats'} = [];
+    $self->{sep} = $sep if defined $sep;
 	#
 	# determine default behavior based either on the setting in
 	# sqlminus, or pre-defined defaults.  Without sqlminus loaded,
@@ -61,14 +64,12 @@ sub header {
 	}
 
 	$self->{feedback}	= $set->{feedback};
-	$self->{limit}		= $set->{limit};
-	$self->{pagesize}	= $set->{pagesize};
-	$self->{recsepchar}	= $set->{recsepchar};
 	$self->{recsep}		= $set->{recsep};
+	$self->{recsepchar}	= $set->{recsepchar};
+	$self->{null}       = $set->{null};
 
-	$self->{pagefeed}	= undef;
-	$self->{pagelen}	= 66;
-	$self->{pagenum}	= 0;
+	$self->{pagesize}	= $set->{pagesize};
+	$self->{limit}		= $set->{limit};
 
 	# $self->{breaks};
 
@@ -151,7 +152,6 @@ sub header {
 
 				if ( exists $cf->{format} and defined $cf->{format} ) {
 					$format_names = $cf->{format};
-					no warnings 'redundant';
 					$width = length sprintf( $format_names, " " );
 				}
 
@@ -292,12 +292,26 @@ sub re_headers {
 sub row {
     my($self, $orig_row) = @_;
     my $i = 0;
-    my @row = $self->SUPER::row([@$orig_row]); # don't alter original
+    my @row = @$orig_row; # don't mess with the original row
 
+# default value for null, is blank.
+	my $null = $self->{'null'} || '';
 	my $columns = $self->{'columns'};
 
 	my $breaks	= $self->{'breaks'};
 
+    map {
+		if (!defined($_)) {
+			$_ = $null;
+		} else {
+			$_ =~ s/\n/\\n/g;
+			$_ =~ s/\t/\\t/g;
+			$_ =~ s/\r/\\r/g;
+			$_ =~ s/[\000-\037\177-\237]/./g;
+		}
+		++$i;
+    } @row;
+
     my $format_rows  = ${$self->{'formats'}};
 
 	# if (exists $self->{'formats'} and defined $self->{'formats'} ){
@@ -370,10 +384,7 @@ sub row {
 		print $fh "\n" x $skip_rows;
 	}
 
-    print $fh form ( 
-		{ 'break'	=> break_with('') }
-		, $format_rows, @data
-		);
+    print $fh form ( { 'break' => break_with('') }, $format_rows, @data);
 
 	++$self->{'rows'};
 
@@ -398,7 +409,6 @@ sub trailer {
 
 	delete $self->{recsep};
 	delete $self->{recsepchar};
-	print "Page Number: ", $self->{pagenum}, "\n";
 
     $self->SUPER::trailer(@_);
 } 
@@ -435,4 +445,4 @@ modify it under the same terms as Perl itself.
 
 =head1 SEE ALSO
 
-L<DBI::Shell>, L<DBI>, L<dbish>
+L<DBI::Shell(3)>, L<DBI(3)>, L<dbish(1)>
