@@ -18,7 +18,10 @@ or
 =head1 DESCRIPTION
 
 The DBI::Shell module (and dbish command, if installed) provide a
-simple but effective command line interface for the Perl L<DBI> module.
+simple but effective command line interface for the Perl DBI module.
+
+DBI::Shell is very new, very experimental and very subject to change.
+Your mileage I<will> vary. Interfaces I<will> change with each release.
 
 =cut
 
@@ -37,9 +40,14 @@ use Carp;
 
 @ISA = qw(Exporter DBI::Shell::Std);
 @EXPORT = qw(shell);
+$VERSION = (qw$Revision: 11.9 $)[1];
 
-our $VERSION = '11.96_03'; # TRIAL VERSION
-$VERSION = eval $VERSION;
+my $warning = <<'EOM';
+
+WARNING: The DBI::Shell interface and functionality are
+=======  very likely to change in subsequent versions!
+
+EOM
 
 sub new {
 	my $class = shift;
@@ -56,12 +64,7 @@ sub new {
 	# Post-init plugins.
 	#$sh->SUPER::init(@args);
     $sh->load_plugins($myconfig->{'plug-ins'}->{'post-init'});
-
-	# do_format is already run in DBI::Shell::Base::new, but that is
-	# before the user config is loaded so we need to run it again if 
-	# the user overrides the format.
-    $sh->do_format($sh->{format});
-	return $sh;
+return $sh;
 }
 
 sub shell {
@@ -76,6 +79,8 @@ sub run {
     die "Unrecognised options: @{$sh->{unhandled_options}}\n"
 	if @{$sh->{unhandled_options}};
 
+    $sh->log($warning) unless $sh->{batch};
+
     # Use valid "dbi:driver:..." to connect with source.
     $sh->do_connect( $sh->{data_source} );
 
@@ -121,15 +126,13 @@ use Carp;
 use Text::Abbrev ();
 use Term::ReadLine;
 use Getopt::Long 2.17;	# upgrade from CPAN if needed: http://www.perl.com/CPAN
+use IO::Tee;
 use IO::File;
-use File::Spec ();
-use File::HomeDir ();
 
 use DBI 1.00 qw(:sql_types :utils);
 use DBI::Format;
 
 use DBI::Shell::FindSqlFile;
-use IO::Interactive qw/ is_interactive /;
 
 use vars qw(@ISA);
 @ISA = qw(DBI::Shell::FindSqlFile);
@@ -137,9 +140,6 @@ use vars qw(@ISA);
 use constant ADD_RH => 1;	# Add the results, to rhistory.
 use constant  NO_RH => 0;	# Do not add results, to rhistory.
 
-# History saving is also provided by DBI::Shell::Completion.
-use constant HISTORY_FILE => '.dbish-builtin-history';
-
 my $haveTermReadKey;
 my $term;
 
@@ -273,12 +273,6 @@ sub default_config {
 	 [ 'tmp_dir|tmp_d=s'	=> $ENV{DBISH_TMP} ],
 	 [ 'tmp_file|tmp_f=s'	=> qq{dbish$$.sql} ],
 	 [ 'home_dir|home_d=s'	=> $ENV{HOME} || "$ENV{HOMEDRIVE}$ENV{HOMEPATH}" ],
-	 [ 'desc_show_remarks|show_remarks' => 1 ],
-	 [ 'desc_show_long|show_long' => 1 ],
-	 [ 'desc_format=s'		=> q{partbox} ],
-	 [ 'desc_show_columns=s' => q{COLUMN_NAME,DATA_TYPE,TYPE_NAME,COLUMN_SIZE,PK,NULLABLE,COLUMN_DEF,IS_NULLABLE,REMARKS} ],
-	 [ 'null_format=s'		=> '(NULL)' ],
-	 [ 'bool_format=s'		=> q{Y,N} ],
 	 @_,
     ) {
 	$sh->add_option(@$opt_ref);
@@ -410,14 +404,11 @@ sub default_term {
     # Setup Term
     #
     my $mode;
-    if (!is_interactive()) {
+    if ($sh->{batch} || ! -t STDIN) {
 		$sh->{batch} = 1;
 		$mode = "in batch mode";
     } else {
 		$sh->{term} = new Term::ReadLine($class);
-		if ($sh->{term}->Features->{readHistory}) {
-			$sh->{term}->ReadHistory(File::Spec->catfile(File::HomeDir->my_home, HISTORY_FILE));
-		}
 		$mode = "";
     }
 
@@ -464,15 +455,11 @@ sub new {
 		}
     }
 
-	# This may be obsolete since it is run again in DBI::Shell::new.
     $sh->do_format($sh->{format});
 
-    $sh->{data_source}  = shift(@args) || $ENV{DBI_DSN}  || '';
-
-    my $user            = shift(@args);
-    $sh->{user}         = defined $user     ? $user     : $ENV{DBI_USER} || '';
-    my $password        = shift(@args);
-    $sh->{password}     = defined $password ? $password : $ENV{DBI_PASS} || undef;
+    $sh->{data_source}	= shift(@args) || $ENV{DBI_DSN}  || '';
+    $sh->{user}		    = shift(@args) || $ENV{DBI_USER} || '';
+    $sh->{password}	    = shift(@args) || $ENV{DBI_PASS} || undef;
 
     $sh->{chistory} = [];	# command history
     $sh->{rhistory} = [];	# result  history
@@ -481,7 +468,7 @@ sub new {
 # set the default io handle.
 	$sh->{out_fh}		= \*STDOUT;
 
-# support for spool command ...
+# support for spool command
 	$sh->{spooling} = 0; $sh->{spool_file} = undef; $sh->{spool_fh} = undef;
 
 	my $mode = $sh->default_term($class);
@@ -547,8 +534,7 @@ sub configuration {
     $sh->{config_file} = $ENV{DBISH_CONFIG} || "$homedir/.dbish_config";
 	my $config;
     if ($sh->{config_file} && -f $sh->{config_file}) {
-		my $full = File::Spec->rel2abs( $sh->{config_file} );
-		$config = require $full;
+		$config = require $sh->{config_file};
 		# allow for custom configuration options.
 		if (exists $config->{'options'} ) {
 			$sh->install_options( $config->{'options'} );
@@ -569,31 +555,15 @@ sub run {
 	$current_line = $sh->readline($sh->prompt());
 	$current_line = "/quit" unless defined $current_line;
 
-	my $copy_cline = $current_line; my $eat_line = 0;
-	# move past command prefix contained within quotes
-	while( $copy_cline =~ s/(['"][^'"]*(?:$prefix).*?['"])//og ) {
-		$eat_line = $+[0];
-	}
-
-	# What's left to check?
-	my $line;
-	if ($eat_line > 0) {
-	    $sh->{current_buffer} .= substr( $current_line, 0, $eat_line ) . "\n";
-		$current_line = substr( $current_line, $eat_line )
-			if (length($current_line) >= $eat_line );
-	} else {
-		$current_line = $copy_cline;
-	}
-
-
 	if ( 
 		$current_line =~ m/
 			^(.*?)
 			(?<!\\)
 			$prefix
+			(?<=[^'"])
 			(?:(\w*)
 			([^\|><]*))?
-			((?:\||>>?|<<?).+)?
+			((?:\||>>?).+)?
 			$
 	/x) {
 	    my ($stmt, $cmd, $args_string, $output) = ($1, $2, $3, $4||''); 
@@ -667,36 +637,22 @@ sub readline {
 sub run_command {
     my ($sh, $command, $output, @args) = @_;
     return unless $command;
-
-    my $code = "do_$command";
-    if ($sh->can("$code")) {
-        local(*STDOUT) if $output;
-        local(*OUTPUT) if $output;
-        if ($output) {
-            if (open(OUTPUT, $output)) {
-                *STDOUT = *OUTPUT;
-            } else {
-                $sh->err("Couldn't open output '$output': $!");
-                $sh->run_command('current', undef, '');
-            }
-        }
-
-       local $@;
-
-       eval {
-            $sh->$code(@args);
-        };
-        close OUTPUT if $output;
-        $sh->err("$command failed: $@") if $@;
-    }
-    else {
-        if ($command eq 'spool') {
-            $sh->err("The DBI::Shell:Spool plug in needs to be installed. See https://rt.cpan.org/Ticket/Display.html?id=24538#txn-813176")
-        }
-        else {
-            $sh->err("$command does not exist, does a plug-in need to be installed?")
-        }
+    local(*STDOUT) if $output;
+    local(*OUTPUT) if $output;
+    if ($output) {
+		if (open(OUTPUT, $output)) {
+			*STDOUT = *OUTPUT;
+		} else {
+			$sh->err("Couldn't open output '$output': $!");
+			$sh->run_command('current', undef, '');
+		}
     }
+    eval {
+		my $code = "do_$command";
+		$sh->$code(@args);
+    };
+    close OUTPUT if $output;
+    $sh->err("$command failed: $@") if $@;	
 return;
 }
 
@@ -926,10 +882,6 @@ sub do_go {
     $sh->push_chistory;
     
     eval {
-		# Determine if the single quotes are out of balance.
-		my $count = ($sh->{current_buffer} =~ tr/'/'/);
-		warn "Quotes out of balance: $count" unless (($count % 2) == 0);
-
 		my $sth = $sh->{dbh}->prepare($sh->{current_buffer});
 
 		$sh->sth_go($sth, 1);
@@ -1088,11 +1040,6 @@ sub do_connect {
 
     $sh->do_disconnect if $sh->{dbh};
 
-	# Change from Jeff Zucker, convert literal slash and letter n to newline.
-	$dsn =~ s/\\n/\n/g;
-	$dsn =~ s/\\t/\t/g;
-
-
     $sh->{data_source} = $dsn;
     if (defined $user and length $user) {
 	$sh->{user}     = $user;
@@ -1143,13 +1090,6 @@ sub do_rollback {
 sub do_quit {
     my ($sh, @args) = @_;
     $sh->do_disconnect if $sh->{dbh};
-
-    if ($sh->{term}) {
-        if ($sh->{term}->Features()->{writeHistory}) {
-            $sh->{term}->WriteHistory(File::Spec->catfile(File::HomeDir->my_home, HISTORY_FILE));
-        }
-    }
-
     undef $sh->{term};
     exit 0;
 }
@@ -1403,121 +1343,46 @@ sub do_describe {
 #	SQL_DATETIME_SUB,CHAR_OCTET_LENGTH,ORDINAL_POSITION,
 #	IS_NULLABLE
 #
-# desc_format: partbox
-# desc_show_long: 1
-# desc_show_remarks: 1
+	push my @desc_cols, qw/TABLE_CAT TABLE_SCHEM TABLE_NAME COLUMN_NAME 
+		DATA_TYPE TYPE_NAME COLUMN_SIZE 
+		DECIMAL_DIGITS NUM_PREC_RADIX NULLABLE 
+		COLUMN_DEF SQL_DATA_TYPE IS_NULLABLE REMARKS/;
 
-	my $schema;
-	if ($tab =~ /^([^.]+)\.([^.]+)$/) {
-		$schema = $1;
-	    $tab = $2;
-	}
-	my @names = ();
-
-	# Determine if the short or long display type is used
-	if (exists $sh->{desc_show_long} and $sh->{desc_show_long} == 1) {
-
-		if (exists $sh->{desc_show_columns} and defined
-			$sh->{desc_show_columns}) {
-			@names = map { defined $_ ? uc $_ : () } split( /[,\s+]/,  $sh->{desc_show_columns});
-			unless (@names) { # List of columns is empty
-				$sh->err ( qq{option desc_show_columns contains an empty list, using default} );
-				# set the empty list to undef
-				$sh->{desc_show_columns} = undef;
-				@names = ();
-				push @names, qw/COLUMN_NAME DATA_TYPE TYPE_NAME COLUMN_SIZE PK
-					NULLABLE COLUMN_DEF IS_NULLABLE/;
-			}
-		} else {
-			push @names, qw/COLUMN_NAME DATA_TYPE TYPE_NAME COLUMN_SIZE PK
-				NULLABLE COLUMN_DEF IS_NULLABLE/;
-		}
-	} else {
-		push @names, qw/COLUMN_NAME TYPE_NAME NULLABLE PK/;
-	}
-
-	# my @names = qw/COLUMN_NAME DATA_TYPE NULLABLE PK/;
-	push @names, q{REMARKS}
-		if (exists $sh->{desc_show_remarks}
-			and $sh->{desc_show_remarks} == 1
-			and (not grep { m/REMARK/i } @names));
-
-	my $sth = $dbh->column_info(undef, $schema, $tab, '%');
+    my $sth = $dbh->column_info(undef, undef, $tab);
 
     if (ref $sth) {
-		
-		# Only attempt the primary_key lookup if using the column_info call.
-
-		my %pk_cols;
-		eval {
-			my @key_column_names = $dbh->primary_key( undef, undef, $tab );
-			# Convert the column names to lower case for matching
-			foreach my $idx (0 ..$#key_column_names) {
-				$pk_cols{lc($key_column_names[$idx])} = $idx;
-			}
-		};
 
 		my @t_data = ();  # An array of arrays
 			
 		while (my $row = $sth->fetchrow_hashref() ) {
 
-			my $col_name	= $row->{COLUMN_NAME};
-			my $col_name_lc	= lc $col_name;
+			push my @out_row, map { $row->{$_} } qw/COLUMN_NAME/;
 
-			# Use the Type name, unless undefined, they use the data type
-			# value.  TODO: Change to resolve the data_type to an ANSI data
-			# type ... SQL_
-			my $type = $row->{TYPE_NAME} || $row->{DATA_TYPE};
+			my $type = $row->{DATA_TYPE};
 
 			if (defined $row->{COLUMN_SIZE}) {
 				$type .= "(" . $row->{COLUMN_SIZE} . ")";
 			}
-			my $is_pk = $pk_cols{$col_name_lc} if exists $pk_cols{$col_name_lc};
 
-			my @out_row;
-			foreach my $dcol (@names) {
-
-				# Add primary key
-				if ($dcol eq q{PK}) {
-					push @out_row, defined $is_pk  ? $is_pk : q{};
-					next;
-				}
-				if ($dcol eq q{TYPE_NAME} and
-					(exists $sh->{desc_show_long} and $sh->{desc_show_long} == 0)) {
-						my $type = $row->{TYPE_NAME} || $row->{DATA_TYPE};
-						if (defined $row->{COLUMN_SIZE}) {
-							$type .= "(" . $row->{COLUMN_SIZE} . ")";
-						}
-					push @out_row, $type;
-					next;
-				}
-
-				# Put a blank if not defined.
-				push @out_row, defined $row->{$dcol} ? $row->{$dcol} :  q{};
-
-				# push(my @out_row
-				# , $col_name
-				# , $type
-				# , sprintf( "%4s", ($row->{NULLABLE} eq 0 ? q{N}: q{Y}))
-				# );
-
-				# push @out_row, defined $row->{REMARKS} ? $row->{REMARKS} :  q{}
-				# 	if (exists $sh->{desc_show_remarks}
-				# 	and $sh->{desc_show_remarks} == 1);
-			}
+			push(@out_row
+				, $type
+				, $row->{NULLABLE} eq 1 ? q{    N}: q{}
+				, q()
+				, $row->{REMARKS} || q{}
+				);
 
 			push @t_data, \@out_row;
 		}
 
 		$sth->finish; # Complete the handler from column_info
 
-
-		# Create a new statement handler from the data and names.
+		my @names = qw/COLUMN_NAME DATA_TYPE NULLABLE PK REMARKS/;
+# Create a new statement handler from the data and names.
 		$sth = $sh->prepare_from_data("describe", \@t_data, \@names);
 
-		# Use the built in formatter to handle data.
+# Use the built in formatter to handle data.
 
-		my $mode = exists $sh->{desc_format} ? $sh->{desc_format} : 'partbox';
+		my $mode = 'partbox';
 		my $class = eval { DBI::Format->formatter($mode,1) };
 		unless ($class) {
 			return $sh->alert("Unable to select '$mode': $@");
@@ -1534,19 +1399,15 @@ sub do_describe {
 			my $rslt = $display->row($rowref);
     	}
 
-    	$display->trailer($i);
+    	return $display->trailer($i);
+
 	}
 
-	#
-	# This is the old method, if the driver doesn't support the DBI column_info
-	# meta data.
-	#
-	$tab = "$schema.$tab" if defined $schema;
 	my $sql = qq{select * from $tab where 1 = 0};
 	$sth = $dbh->prepare( $sql );
 	$sth->execute;
 	my $cnt = $#{$sth->{NAME}};  #
-    @names = qw{NAME TYPE NULLABLE};
+    my @names = qw{NAME TYPE NULLABLE};
 	my @ti;
 	for ( my $c = 0; $c <= $cnt; $c++ ) {
 		push( my @j, $sth->{NAME}->[$c] || 0 );
@@ -1661,10 +1522,93 @@ sub do_table_info {
 return $sh->sth_go($sth, 0, NO_RH);
 }
 
+#------------------------------------------------------------------
+#
+# Start or Stop spooling output.
+# The spool support the follow states:
+# spool - returns the current state of spooling, if on includes the file name.
+# spool on - set the state to on, opens a default name of spool.lst (Yes, the
+# Oracle default name).  If the spool current state is already on, returns a
+# warning message (Already spooling to file X).
+# spool /path/file/name - set the state on, attempt to open the file name
+# (using the IO::Tee object to allow multiplex output), and set the new IO
+# handle to the default handle.
+# spool off - set the state to off.  If the previous state was on, flush the
+# current buffer and close the file handle.  If the previous state was off,
+# return a warning message (Not current spooling).
+#
+#------------------------------------------------------------------
 # Support functions.
-sub is_spooling	( ) { return shift->{spooling}		}
-sub spool_on	( ) { return shift->{spooling} = 1	}
-sub spool_off	( ) { return shift->{spooling} = 0	}
+sub is_spooling( ) { return shift->{spooling} }
+sub spool_on ( ) { return shift->{spooling} = 1 }
+sub spool_off( ) { return shift->{spooling} = 0 }
+
+sub do_spool {
+	my ($sh, @args) = @_;
+
+# Get the current state of spool.
+	unless(@args) {
+		if ($sh->is_spooling) {
+			return $sh->print_buffer( qq{spooling output to file: },
+			$sh->{spool_file} );
+		} else {
+			return $sh->print_buffer( qq{not spooling} );
+		}
+	}
+
+# So what command did I get at this point?
+	my $command = shift @args;
+
+	if ($command =~ m/\boff/i) {	# Turn the spool off (if on).
+		if ($sh->is_spooling) { # spool on
+			# The tee object contains the open handles, get a list, shift the
+			# first (this should be STDOUT), flush.  Then for the remainder
+			# flush each and close.
+			my @fhs = $sh->{out_fh}->handles;
+			$sh->{out_fh} = shift @fhs; select $sh->{out_fh};
+
+			$sh->{out_fh}->flush;
+			$sh->spool_off; $sh->{spool_file} = undef;
+			foreach my $fh (@fhs) {
+				$fh->flush;
+				$fh->close;
+			}
+			$sh->{spool_fh} = undef;
+			return $sh->{out_fh};
+		}
+		return $sh->print_buffer( qq{not spooling} );
+	}
+
+	my $spool_file = undef;
+	if ($command =~ m/on/i) {	# Turn the spool off (if on).
+		unless(@args or $args[0] !~ m/!/) {
+			$spool_file = q{on.lst};
+		}
+	}
+
+	# OK, now we're at the one to open the spool file.  How do I handle if the
+	# file exists? Well, unless the next arg is a !, open the file for append.
+	my $mode = q{a+};
+	if (@args and $args[0] =~ m/!/) {
+		shift @args; 
+		$mode = q{w};
+	}
+	
+	my $out_fh		= $sh->{out_fh};
+
+	$spool_file = defined $spool_file ? $spool_file : $command;
+
+	if (defined $spool_file) {
+		my $tee_fh = new IO::Tee($out_fh, new IO::File($spool_file, $mode)) or
+			return $sh->alert(qq{Unable create IO::Tee ($spool_file) handle: $!\n});
+		$sh->{out_fh} = $tee_fh;
+		$sh->{spool_file} = $spool_file; $sh->spool_on;
+		$sh->{spool_fh} = ($tee_fh->handles)[1];
+		select $tee_fh;
+		return $sh->print_buffer( qq{spooling $spool_file} );
+	}
+return $sh->alert( qq{spool command failed for unknown reason} );
+}
 
 1;
 __END__
@@ -1885,60 +1829,6 @@ Useful for determining what is really happening in DBI.  See DBI.
 
 =back
 
-
-=head1 CONFIGURATION
-
-The configuration file (F<~/.dbish_config> by default, overridden by
-C<$DBISH_CONFIG>) is eval'd as perl code and should return a hash ref
-containing the configuration items. Notable keys include
-
-=over
-
-=item options
-
-The value is another hashref containing the options and their values.
-Type C</option> at the prompt to get a list of acceptable options.
-
-=item plug-ins
-
-Plugins to load. This is a rather complex data structure which is
-recursively walked to determine which plugins should be loaded. At the
-top level, there are the phases
-C<pre-init>, C<post-init>, C<pre-connect>, and C<post-connect>. At lower
-levels plugins can be selected by the name of the database driver, and
-individual options can be set. Finally, at some point an arrayref
-containing the package names of the plugins to load must be found.
-
-=back
-
-Since the config file can contain arbitrary perl code, it can do
-anything which can be done in perl. One rather useful hack is to call
-binmode to set the encoding for standard input and output. 
-
-=head2 Example
-
-    binmode STDIN,  ":utf8";
-    binmode STDOUT, ":utf8";
-
-    {
-        options => {
-            format => 'partbox',
-        },
-        'plug-ins' => {
-            'pre-init' => [
-                'DBI::Shell::Timing',
-                'DBI::Shell::Spool',
-            ],
-        },
-    };
-
-This config file first sets the :utf8 layer for input and output to
-match the terminal (if you don't use UTF-8, you could use :encoding()
-here), and then returns the hashref with the configuration items:
-The format is set 'partbox', and the the two plugins
-'DBI::Shell::Timing' and 'DBI::Shell::Spool' are loaded.
-
-
 =head1 ENVIRONMENT
 
 =over 4
@@ -1960,7 +1850,7 @@ configurations.
 The DBI::Shell has a long lineage.
 
 It started life around 1994-1997 as the pmsql script written by Andreas
-KE<ouml>nig. Jochen Wiedmann picked it up and ran with it (adding much along
+Knig. Jochen Wiedmann picked it up and ran with it (adding much along
 the way) as I<dbimon>, bundled with his DBD::mSQL driver modules. In
 1998, around the time I wanted to bundle a shell with the DBI, Adam
 Marks was working on a dbish modeled after the Sybase sqsh utility.
@@ -1975,8 +1865,7 @@ from Jochen's dbimon is bound to find it's way back in.
 
 The DBI::Shell module is Copyright (c) 1998 Tim Bunce. England.
 All rights reserved. Portions are Copyright by Jochen Wiedmann,
-Adam Marks, Tom Lowery, Kent Fredric, Rafael Kitover and Mike Pomraning. It
-is currently maintained by Dave Lambley.
+Adam Marks and Tom Lowery.
 
 You may distribute under the terms of either the GNU General Public
 License or the Artistic License, as specified in the Perl README file.
