@@ -2,9 +2,6 @@
 
 package DBI::Shell::FindSqlFile;
 
-our $VERSION = '11.96_03'; # TRIAL VERSION
-$VERSION = eval $VERSION;
-
 use strict;
 use File::Find ();
 use File::Basename;
@@ -48,8 +45,7 @@ sub look_for_file {
 	$self->log("calling find with $to_find_file") if $self->{debug};
 
 
-# Split the sqlpath, then determine if any of the directories are valid.
-	my @search_path = map { -d $_ ? $_ : () } split(/:/,
+	my @search_path = split(/:/,
 		   defined $self->{sqlpath} ? $self->{sqlpath} : ()
 		   );
 		# ,  (exists $ENV{DBISH_SQL_PATH} ?  $ENV{DBISH_SQL_PATH} : ()) );
@@ -61,7 +57,7 @@ sub look_for_file {
 	File::Find::find(
 		{
 			  wanted 	=> \&wanted
-			, no_chdir 	=> 1
+			, no_chrdir 	=> 1
 			, bydepth	=> 0
 		}, 
 		@search_path);
