@@ -3,13 +3,12 @@
 #
 package DBI::Shell::Timing;
 
-our $VERSION = '11.96_03'; # TRIAL VERSION
-$VERSION = eval $VERSION;
-
 use strict;
-use vars qw(@ISA);
+use vars qw(@ISA $VERSION);
 use Benchmark qw(timeit timestr);
 
+$VERSION = (qw$Revision: 11.6 $)[1];
+
 sub init {
 	my ($self, $sh, @arg)  = @_;
 
